#include <errno.h>
#include <fcntl.h>
#include <stdbool.h>
#include <string.h>
#include <termios.h>
#include "NcPrintErr.h"
#include "NcUart.h"

bool UartConfigure(int fd, int speed, int rxTimeOutMs)
{
	int parity = 0;
    struct termios tty;
    memset(&tty, 0, sizeof tty);
    if (tcgetattr(fd, &tty) != 0)
    	return PrintErr("tcgetattr failed. errno=%s", PrintErrNo(errno));
    cfsetospeed (&tty, speed);
    cfsetispeed (&tty, speed);
    tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS8;     // 8-bit chars
    tty.c_iflag &= ~IGNBRK;         // disable break processing
    tty.c_lflag = 0;                // no signaling chars, no echo,
                                    // no canonical processing
    tty.c_oflag = 0;                // no remapping, no delays
    tty.c_cc[VMIN]  = 0;            // read doesn't block (http://www.unixwiz.net/techtips/termios-vmin-vtime.html)
    tty.c_cc[VTIME] = rxTimeOutMs * 10/1000; // resolution is 10th of a second
    tty.c_iflag &= ~(IXON | IXOFF | IXANY); // shut off xon/xoff ctrl
    tty.c_cflag |= (CLOCAL | CREAD);// ignore modem controls, // enable reading
    tty.c_cflag &= ~(PARENB | PARODD);      // shut off parity
    tty.c_cflag |= parity;
    tty.c_cflag &= ~CSTOPB;
    tty.c_cflag &= ~CRTSCTS;
	
	tty.c_oflag&=~(ONLCR|OCRNL);
	tty.c_iflag&=~(INLCR|ICRNL);
	
	cfmakeraw(&tty);
	
    if (tcsetattr(fd, TCSANOW, &tty) != 0)
    	return PrintErr("tcsetattr failed. errno=%s", PrintErrNo(errno));
    return true;
}

bool UartOpen(const char * uart, int * pFd)
{
	int fd = open( uart, O_RDWR | O_NOCTTY | O_SYNC);
	if (fd<0)
		return PrintErr("Failed to open UART %s. errno=%s", uart, PrintErrNo(errno));
	*pFd = fd;
	return true;
}
