#include <errno.h>
#include <fcntl.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "NcClosers.h"
#include "Configuration.h"
#include "CtsTask.h"
#include "NcApi.h"
#include "NcPrintErr.h"

CtsState g_cts;

#ifdef __cplusplus
extern "C"
{
#endif
extern void CtsMainIoctl();
extern void CtsMainGpio();
#ifdef __cplusplus
}
#endif

bool CtsTaskInitInternal(Configuration * c)
{
	memset(&g_cts, 0, sizeof(g_cts));
	g_cts.config = c;
	const char * cs = c->getCtsSource();
	if (strcmp(cs, CONFIG_CTSSOURCE_GPIO)==0)
	{
		g_cts.pfnMain = CtsMainGpio;
	}
	else if (strcmp(cs, CONFIG_CTSSOURCE_IOCTL)==0)
	{
		g_cts.ioctlFd = open( c->getUart(), O_RDWR );
		if (g_cts.ioctlFd<0)
			return PrintErr("CtsTask failed to open UART. errno=%s", PrintErrNo(errno));
		g_cts.pfnMain = CtsMainIoctl;
	}
	else
		return PrintErr("Invalid ctsSource: %s", cs);
	return true;
}

bool CtsTaskInit(void * p)
{
	Configuration * c = (Configuration*)p;
	return CtsTaskInitInternal(c);
}

void* CtsTaskRun(void * p)
{
	g_cts.pfnMain();
	return NULL;
}

bool CtsTaskShutdown()
{
	FdClose(&g_cts.ioctlFd);
	FdClose(&g_cts.gpioFd);
	return false;
}
