#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
#include <iostream>
#include <vector>
#include <netinet/ip.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include "rapidjson/writer.h"
#include "rapidjson/stringbuffer.h"
#include "NcClosers.h"
#include "Configuration.h"
#include "Constants.h"
#include "NcApi.h"
#include "NcPrintErr.h"
#include "RecvTask.h"
#include "NcSocket.h"
#include "NcUart.h"
#include <poll.h>

#include "RecvEssentials.h"


bool ConfigureUart(tRecvConfig * c)
{
	return UartConfigure(g_recv.fd, c->getSpeed(), UART_READ_TIMEOUT_MS);
}

bool OpenSocket(tRecvConfig * c)
{
	return SocketOpenServer(c->getRecvPort(), &g_recv.socketFd, &g_recv.address);
}

bool OpenUart(tRecvConfig * c)
{
	return UartOpen(c->getUart(), &g_recv.fd);
}

bool RecvTaskInitInternal(Configuration * c)
{
	RecvTaskInitEssentials(c);
	g_recv.config = c;
	if (!OpenUart(c) || !OpenSocket(c))
		return false;
	g_recv.state = Started;
	return true;
}

bool RecvTaskInit(void * p)
{
	Configuration * c = (Configuration *)p;
	return RecvTaskInitInternal(c);
}

bool RecvTaskShutdown()
{
	g_recv.state = Stopped;
	return true;
}

void RecvTaskCloseUart()
{
	FdClose(&g_recv.fd);
}

void RecvTaskCloseSockets()
{
	FdClose(&g_recv.socketFd);
	size_t i;
	for (i=0; i<g_recv.clients.size(); i++)
		close(g_recv.clients.at(i));
	g_recv.clients.clear();
}

void RecvTaskClose()
{
	RecvTaskCloseUart();
	RecvTaskCloseSockets();
}


int RxCount=0;
int RxCountError=0;

void RecvTaskPumpUart()
{
	uint8_t buf[ UART_READ_BUFFER ];
	int len;
	struct pollfd fds[1];
	int ret;
    int timeout_msecs = 100;	
	fds[0].fd = g_recv.fd;
	fds[0].events = POLLIN;
	

	ret = poll(fds, 1, timeout_msecs);
	if (ret==0)
	{
		// Timeout, resync serial
		NcApiCallbackNwuActive(0);
		if (RxCount!=0)
		{
			//printf("RxCount Error!!!!\n");
			RxCountError++;
			
		}
		RxCount=0;
		
	}
	else if (ret>0)
	{
		if ((fds[0].revents & POLLIN)!= 0)
		{
			if ((len = read( g_recv.fd, buf, sizeof(buf)))>0)
			{
				for (int i=0; i<len; i++)
				{
					RxCount++;
					NcApiRxData(0, buf[i]);
				}
			}
		}
	}
}

void RecvTaskAcceptSockets()
{
	int clientFd;
	while (SocketAcceptNonBlock(g_recv.socketFd, &clientFd))
		g_recv.clients.push_back(clientFd);
}

void RecvTaskLoop()
{
	RecvTaskPumpUart();
	RecvTaskAcceptSockets();
}

void* RecvTaskRun(void* p)
{
	tRecvConfig * c = (tRecvConfig *)p;
    if (!ConfigureUart(c))
    	return NULL;
	g_recv.state = Started;
	while(true)
	{
		switch (g_recv.state)
		{
		case Stopped:
			RecvTaskClose();
			return NULL;
		case Started:
			RecvTaskLoop();
			break;
		case ReLoadConfig:
			RecvTaskClose();
			g_recv.state = RecvTaskInit(g_recv.config)
					? Started
					: Stopped;
			break;
		}
	}
	return NULL;
}


void DispatchFormattedMsg(const char * m)
{
	printf("-->%s\n", m);
	const char * del = g_recv.config->getDelimiterFormatted();
	int delLen = strlen(del);
	int len = strlen(m);
	int finalLen = len + delLen;
	char * final = (char *)malloc(finalLen);
	memcpy(final, m, len);
	memcpy(final+len, del, delLen);
	std::vector<int> newClients;
	size_t i;
	size_t clientsLen = g_recv.clients.size();
	if (clientsLen>0)
	{
		for (i=0; i<clientsLen; i++)
		{
			int clientFd = g_recv.clients.at(i);
			if (send(clientFd, final, finalLen, MSG_NOSIGNAL)==finalLen)
				newClients.push_back(clientFd);
		}
	}
	free((void*)m);
	free((void*)final);
	g_recv.clients = newClients;
}

void UartReadCallbackSpecialized(uint8_t n, uint8_t * msg, uint8_t msgLength)
{
	RxCount=0;
	//printf("RxCount Error %d\n",RxCountError);
	
	UartReadCallback(n, msg, msgLength);
}

void NcApiSupportMessageReceived(uint8_t n,void * callbackToken, uint8_t * msg, uint8_t msgLength)
{
	NcApiExecuteCallbacks(n,msg,msgLength);
}

