#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include "SendEssentials.h"

#ifdef GATEWAY_DEBUG_INFORMATION
#include "Helpers.h"
#endif


SendState g_send;


void SendTaskInitEssentials(ConfigEssentials * c)
{
	memset(&g_send, 0, sizeof(g_send));
	g_send.config_ser = c;
}

/**
 * \brief Parse a JSON string to be sent as an unacknowledged command
 * This function takes its input from the global structure "g_send"
 * @return A string that indicates any error; NULL if everything is OK
 */
const char* ParseMsgSendUnackParams()
{
#ifdef GATEWAY_DEBUG_INFORMATION
	printf("     ParseMsgSendUnackParams \r\n");
#endif
	memset(&g_send.ncSendUnackParams, 0, sizeof(g_send.ncSendUnackParams));
	tNcApiSendUnackMessage m;
	const char * ret = g_send.config_ser->getSerializer()->DeSerialize((const char*)g_send.clientRaw, g_send.clientRawLen, &m);
	
	if (ret!=NULL) return ret;
	g_send.ncApiType = CommandUnacknowledgedEnum;
	g_send.ncSendUnackParams.callbackToken = NULL;
	g_send.ncSendUnackParams.msg.destNodeId = m.destNodeId;
	g_send.ncSendUnackParams.msg.destPort = m.destPort;
	g_send.ncSendUnackParams.msg.appSeqNo = m.appSeqNo;
	g_send.ncSendUnackParams.msg.payload = m.payload;
	g_send.ncSendUnackParams.msg.payloadLength = m.payloadLength;

	return NULL;
}

/**
 * \brief Parse a JSON string to be sent as an acknowledged command
 * This function takes its input from the global structure "g_send"
 * @return A string that indicates any error; NULL if everything is OK
 */
const char* ParseMsgSendAckParams()
{
#ifdef GATEWAY_DEBUG_INFORMATION
	printf("     ParseMsgSendAckParams \r\n");
#endif
	memset(&g_send.ncSendAckParams, 0, sizeof(g_send.ncSendAckParams));
	tNcApiSendAckMessage m;
	const char * ret = g_send.config_ser->getSerializer()->DeSerialize((const char*)g_send.clientRaw, g_send.clientRawLen, &m);
	
	if (ret!=NULL) return ret;
	g_send.ncApiType = CommandAcknowledgedEnum;
	g_send.ncSendAckParams.callbackToken = NULL;
	g_send.ncSendAckParams.msg.destNodeId = m.destNodeId;
	g_send.ncSendAckParams.msg.destPort = m.destPort;
	g_send.ncSendAckParams.msg.payload = m.payload;
	g_send.ncSendAckParams.msg.payloadLength = m.payloadLength;

	return NULL;
}

const char* ParseMsgNodeInfoRequest()
{
#ifdef GATEWAY_DEBUG_INFORMATION
	printf("     ParseMsgNodeInfoRequest \r\n");
#endif
	memset(&g_send.ncNodeInfoParams, 0, sizeof(g_send.ncNodeInfoParams));
	tNcApiNodeInfoRequestMessage m;
	const char * ret = g_send.config_ser->getSerializer()->DeSerialize((const char*)g_send.clientRaw, g_send.clientRawLen, &m);

	if (ret!=NULL) return ret;
	g_send.ncApiType = NodeInfoRequestEnum;

	return ret;
}

const char* ParseMsgNeighborListRequest()
{
#ifdef GATEWAY_DEBUG_INFORMATION
	printf("     ParseMsgNeighborRequest \r\n");
#endif
	memset(&g_send.ncNeighborListParams, 0, sizeof(g_send.ncNeighborListParams));
	tNcApiNeighborListRequestMessage m;
	const char * ret = g_send.config_ser->getSerializer()->DeSerialize((const char*)g_send.clientRaw, g_send.clientRawLen, &m);

	if (ret!=NULL) return ret;
	g_send.ncApiType = NeighborListRequestEnum;

	return ret;
}

const char* ParseMsgNetCmdRequest()
{
#ifdef GATEWAY_DEBUG_INFORMATION
	printf("     ParseMsgNetCmdRequest \r\n");
#endif
	memset(&g_send.ncNetCmdParams, 0, sizeof(g_send.ncNetCmdParams));
	tNcApiNetCmdMessage m;
	const char * ret = g_send.config_ser->getSerializer()->DeSerialize((const char*)g_send.clientRaw, g_send.clientRawLen, &m);

	if (ret!=NULL) return ret;
	g_send.ncApiType = NetCmdEnum;
	g_send.ncNetCmdParams.callbackToken = NULL;
	g_send.ncNetCmdParams.msg.destNodeId = m.destNodeId;
	g_send.ncNetCmdParams.msg.cmd = m.cmd;
	g_send.ncNetCmdParams.msg.payload = m.payload;
	g_send.ncNetCmdParams.msg.payloadLength = m.payloadLength;

	return ret;
}

const char* ParseMsgWesCmd()
{
#ifdef GATEWAY_DEBUG_INFORMATION
	printf("     ParseMsgWesCmd \r\n");
#endif
	memset(&g_send.ncWesCmd, 0, sizeof(g_send.ncWesCmd));
	tNcApiWesCmdMessage m;
	const char * ret = g_send.config_ser->getSerializer()->DeSerialize((const char*)g_send.clientRaw, g_send.clientRawLen, &m);

	if (ret!=NULL) return ret;
	g_send.ncApiType = WesCmdEnum;
	g_send.ncWesCmd.msg.cmd = m.cmd;

	return ret;
}

const char* ParseMsgWesResponse()
{
#ifdef GATEWAY_DEBUG_INFORMATION
	printf("     ParseMsgWesResponse \r\n");
#endif
	memset(&g_send.ncWesResponse, 0, sizeof(g_send.ncWesResponse));
	tNcApiWesResponseMessage m;
	const char * ret = g_send.config_ser->getSerializer()->DeSerialize((const char*)g_send.clientRaw, g_send.clientRawLen, &m);

	if (ret!=NULL) return ret;
	g_send.ncApiType = WesResponseEnum;
	g_send.ncWesResponse.msg.nodeId = m.nodeId;
	memcpy( g_send.ncWesResponse.msg.uid, m.uid, 5 );
	memcpy( g_send.ncWesResponse.msg.appSettings, m.appSettings, WES_APPSETTINGS_LENGTH );

	return ret;
}

const char* ParseMsgAltCmd()
{
#ifdef GATEWAY_DEBUG_INFORMATION
	printf("     ParseMsgAltCmd \r\n");
#endif
	memset(&g_send.ncAltCmd, 0, sizeof(g_send.ncAltCmd));
	tNcApiAltCmdMessage m;
	const char * ret = g_send.config_ser->getSerializer()->DeSerialize((const char*)g_send.clientRaw, g_send.clientRawLen, &m);

	if (ret!=NULL) return ret;
	g_send.ncApiType = AltCmdEnum;
	g_send.ncAltCmd.msg.cmd = m.cmd;

	return ret;
}


/**
 * \brief Parse a JSON string
 *
 * The input string is parsed to verify that it is a valid JSON string.
 * The called functions take their input from the global structure "g_send",
 * and if this is the case the "g_send.ncApiType" is set accordingly.
 *
 * @return A string that indicates any error; NULL if everything is OK
 */
const char* ParseMsg()
{
#ifdef GATEWAY_DEBUG_INFORMATION
	printf("    ParseMsg \r\n");
#endif
	const char *    ret = ParseMsgSendUnackParams();
	if (ret!=NULL ) ret = ParseMsgSendAckParams();
	if (ret!=NULL ) ret = ParseMsgNodeInfoRequest();
	if (ret!=NULL ) ret = ParseMsgNeighborListRequest();
	if (ret!=NULL ) ret = ParseMsgNetCmdRequest();
	if (ret!=NULL ) ret = ParseMsgWesCmd();
	if (ret!=NULL ) ret = ParseMsgWesResponse();
	if (ret!=NULL ) ret = ParseMsgAltCmd();
	return ret;
}

void PrintMsg()
{
	uint8_t * p = g_send.clientRaw;
	int len = g_send.clientRawLen;
	p[len] =  0;

	printf("Data received: \r\n<--%s\r\n", p);
}

NcApiErrorCodes SendMessageToNcApi()
{
#ifdef GATEWAY_DEBUG_INFORMATION
	printf("    SendMessageToNcApi \r\n");
#endif
	g_send.msgWasWritten = false;
	NcApiErrorCodes ncRet = NCAPI_OK;
	switch (g_send.ncApiType)
	{
		case CommandUnacknowledgedEnum:
			ncRet = NcApiSendUnacknowledged(0, &g_send.ncSendUnackParams);
#ifdef GATEWAY_DEBUG_INFORMATION
			NcApiSendUnacknowledged_DebugHelper(&g_send.ncSendUnackParams);
#endif
			break;

		case CommandAcknowledgedEnum:
			ncRet = NcApiSendAcknowledged(0, &g_send.ncSendAckParams);
#ifdef GATEWAY_DEBUG_INFORMATION
			NcApiSendAcknowledged_DebugHelper(&g_send.ncSendAckParams);
#endif
			break;

		case NodeInfoRequestEnum:
			ncRet = NcApiSendNodeInfoRequest(0, &g_send.ncNodeInfoParams);
#ifdef GATEWAY_DEBUG_INFORMATION
			NcApiSendNodeInfoRequest_DebugHelper(&g_send.ncNodeInfoParams);
#endif
			break;

		case NeighborListRequestEnum:
			ncRet = NcApiSendNeighborListRequest(0, &g_send.ncNeighborListParams);
#ifdef GATEWAY_DEBUG_INFORMATION
			NcApiSendNeighborListRequest_DebugHelper(&g_send.ncNeighborListParams);
#endif
			break;

		case NetCmdEnum:
			ncRet = NcApiSendNetCmd(0, &g_send.ncNetCmdParams);
#ifdef GATEWAY_DEBUG_INFORMATION
			NcApiSendNetCmd_DebugHelper(&g_send.ncNetCmdParams);
#endif
			break;

		case WesCmdEnum:
			ncRet = NcApiSendWesCmd(0, &g_send.ncWesCmd);
#ifdef GATEWAY_DEBUG_INFORMATION
			NcApiSendWesCmd_DebugHelper(&g_send.ncWesCmd);
#endif
			break;

		case WesResponseEnum:
			ncRet = NcApiSendWesResponse(0, &g_send.ncWesResponse);
#ifdef GATEWAY_DEBUG_INFORMATION
			NcApiSendWesResponse_DebugHelper(&g_send.ncWesResponse);
#endif
			break;

		case AltCmdEnum:
			ncRet = NcApiSendAltCmd(0, &g_send.ncAltCmd);
#ifdef GATEWAY_DEBUG_INFORMATION
			NcApiSendAltCmd_DebugHelper(&g_send.ncAltCmd);
#endif
			break;
			
		default:
			break;
	}
	return ncRet;
}
